@layer wf{.BuilderInsertionLabel[data-v-9cd536ed]{display:flex;align-items:center;justify-content:center;overflow:hidden;white-space:nowrap;box-shadow:0 0 8px -1px #0009;position:absolute;border-radius:18px;padding:0 16px;height:36px;font-size:.75rem;color:var(--builderBackgroundColor);background:var(--builderIntenseSelectedColor)}}
