@layer wf{.BuilderInsertionOverlay[data-v-0c332d4f]{min-width:100%;min-height:100%;pointer-events:none;position:relative;display:flex;justify-content:start}.overlay[data-v-0c332d4f]{width:100%;height:100%;position:absolute;opacity:.8;top:0;left:0;animation:.2s ease-in appear-0c332d4f forwards}.progressBar[data-v-0c332d4f]{position:absolute;top:0;left:0;height:4px;width:100%;background:var(--builderSeparatorColor)}.progressBar .elapsed[data-v-0c332d4f]{height:4px;background:var(--builderIntenseSelectedColor);animation:linear elapse-0c332d4f forwards}@keyframes appear-0c332d4f{0%{background-color:transparent}to{background-color:var(--builderSelectedColor)}}@keyframes elapse-0c332d4f{0%{width:0}to{width:100%}}}
