@layer wf{h1[data-v-ec78cfd2],h2[data-v-ec78cfd2],h3[data-v-ec78cfd2]{font-weight:600;margin:0}h1[data-v-ec78cfd2]{font-size:1.3rem}h2[data-v-ec78cfd2]{font-size:1rem}h3[data-v-ec78cfd2]{font-size:14px;font-style:normal;font-weight:600}h4[data-v-ec78cfd2]{text-transform:uppercase;font-weight:700;font-size:.65rem;letter-spacing:.2ch}.insertionOverlay[data-v-ec78cfd2]{background:#d2eaf480;pointer-events:none;position:absolute;top:0;left:0;max-height:100%;max-width:100%}.insertionOverlay .label[data-v-ec78cfd2]{border-radius:16px;box-shadow:0 0 8px -1px #0006;position:absolute;right:0;height:24px;padding:4px 16px 0;top:-24px;color:#fff;background:var(--builderActionOngoingColor)}button[data-v-ec78cfd2]{border:none;background:var(--builderSubtleSeparatorColor);outline:none;color:var(--builderPrimaryTextColor);padding:10px 12px;border-radius:4px;font-size:.75rem;cursor:pointer;display:flex;gap:8px;align-items:center}button[data-v-ec78cfd2]:has(i+i){gap:0}button[variant=subtle][data-v-ec78cfd2]{background:unset}button[variant=subtle][data-v-ec78cfd2]:hover,button[variant=subtle][data-v-ec78cfd2]:focus{background:var(--builderActionOngoingColor);color:var(--builderBackgroundColor)}textarea[variant=code][data-v-ec78cfd2]{font-family:Consolas,monospace}input[type=button][data-v-ec78cfd2],input[type=submit][data-v-ec78cfd2],input[type=radio][data-v-ec78cfd2],input[type=reset][data-v-ec78cfd2],input[type=color][data-v-ec78cfd2]{cursor:pointer}code[data-v-ec78cfd2]{font-family:monospace}.markdown code[data-v-ec78cfd2]{background-color:var(--builderSubtleSeparatorColor);padding:2px}.BuilderSettingsHandlers[data-v-ec78cfd2]{padding:24px}.BuilderSettingsHandlers__title[data-v-ec78cfd2]{padding-bottom:24px;display:flex;gap:8px;align-items:center;font-size:1rem}.BuilderSettingsHandlers__list[data-v-ec78cfd2]{display:flex;flex-direction:column;gap:24px}.addHandler .fields[data-v-ec78cfd2]{display:flex;gap:16px;margin-bottom:16px}.stubMessage[data-v-ec78cfd2]{font-size:.8rem}.codeContainer[data-v-ec78cfd2]{white-space:pre-wrap;background:var(--builderSubtleHighlightColorSolid);padding:16px;border-radius:4px;font-family:Consolas,monospace;font-size:.85rem;margin-top:12px;margin-bottom:12px;overflow:hidden;text-overflow:ellipsis}.codeContainer code[data-v-ec78cfd2]{white-space:pre-wrap}.customHandlerModalForm[data-v-ec78cfd2]{display:grid;grid-template-columns:1fr 1fr;align-items:center;gap:16px;margin-bottom:16px}}
