@layer wf{.CoreChatbotLoader[data-v-06a14934]{width:100px;height:100%;display:flex;gap:var(--dotSize);align-items:center;justify-content:center}.CoreChatbotLoader__dot[data-v-06a14934]{display:inline-block;width:var(--dotSize);height:var(--dotSize);position:relative}.CoreChatbotLoader__dot[data-v-06a14934]:after{display:inline-block;content:"";position:absolute;background-color:currentColor;width:calc(var(--dotSize) / 2);height:calc(var(--dotSize) / 2);border-radius:50%;animation:changeSize-06a14934 1.5s infinite;top:50%;left:50%;transform:translate(-50%,-50%)}.CoreChatbotLoader__dot[data-v-06a14934]:nth-child(1):after{animation-delay:0s}.CoreChatbotLoader__dot[data-v-06a14934]:nth-child(2):after{animation-delay:.5s}.CoreChatbotLoader__dot[data-v-06a14934]:nth-child(3):after{animation-delay:1s}@keyframes changeSize-06a14934{0%{width:calc(var(--dotSize) / 2);height:calc(var(--dotSize) / 2)}50%{width:var(--dotSize);height:var(--dotSize)}to{width:calc(var(--dotSize) / 2);height:calc(var(--dotSize) / 2)}}}
