import writer as wf
import writer.ai

# Welcome to Writer Framework! 
# This template is a starting point for your AI apps.
# More documentation is available at https://dev.writer.com/framework


# Initialise the state
wf.init_state({
    "my_app": {
        "title": "AI STARTER"
    },
})