@layer wf{.BuilderSidebarPanel[data-v-99679ce7]{display:grid;grid-template-rows:auto 1fr;grid-template-columns:100%;height:100%;width:100%;position:relative;overflow-x:auto;overflow-y:scroll}.BuilderSidebarPanel__inputContainer[data-v-99679ce7]{position:sticky;top:0;left:0;right:0;padding:16px 16px 0;background:var(--builderBackgroundColor);z-index:2}.searchInput[data-v-99679ce7]{background:var(--builderSubtleSeparatorColor);width:100%}.BuilderSidebarPanel__main[data-v-99679ce7]{display:flex;padding:16px;gap:16px;flex-direction:column}.BuilderSidebarPanel__footer[data-v-99679ce7]{position:sticky;bottom:0;left:0}.categories[data-v-99679ce7]{display:flex}.category .header[data-v-99679ce7]{font-size:12px;font-weight:500;line-height:12px;letter-spacing:1.3px;text-transform:uppercase;color:var(--builderSecondaryTextColor);margin-bottom:16px}.tools[data-v-99679ce7]{display:grid;grid-template-columns:24px 1fr;grid-template-rows:auto;padding:0 8px;row-gap:12px;column-gap:4px}}@layer wf{.category .header[data-v-be3d3864]{font-size:12px;font-weight:500;line-height:12px;letter-spacing:1.3px;text-transform:uppercase;color:var(--builderSecondaryTextColor);margin-bottom:8px}.tools[data-v-be3d3864]{display:grid;grid-template-columns:1fr;grid-template-rows:auto}.tool[data-v-be3d3864]{display:grid;grid-template-columns:18px 1fr;grid-template-rows:1fr;column-gap:8px;padding:8px;border-radius:4px;cursor:grab}.tool img[data-v-be3d3864]{max-width:18px;max-height:18px;aspect-ratio:1 / 1}.tool[data-v-be3d3864]:hover{background:var(--builderSubtleSeparatorColor)}}@layer wf{.BuilderSidebarComponentTreeBranch[data-v-7764053f]{min-width:170px}.BuilderSidebarComponentTreeBranch__nameRight[data-v-7764053f]{display:flex;align-items:center;gap:4px;flex-wrap:nowrap;text-wrap:nowrap;text-overflow:ellipsis;min-width:0}.previewText[data-v-7764053f]{flex-wrap:nowrap;text-wrap:nowrap;flex-grow:1;white-space:nowrap;text-overflow:ellipsis;overflow:hidden}.middot[data-v-7764053f]{display:inline-block;border-radius:3px;width:3px;min-width:3px;height:3px;background:var(--builderSecondaryTextColor);opacity:.5}}@layer wf{.BuilderSidebarComponentTree[data-v-1b58d296]{height:100%;position:relative}.category .header[data-v-1b58d296]{font-size:12px;font-weight:500;line-height:12px;letter-spacing:1.3px;text-transform:uppercase;color:var(--builderSecondaryTextColor);margin-bottom:16px}.rootBranch[data-v-1b58d296]{margin-top:-8px;flex:1 0 auto}.tools[data-v-1b58d296]{display:grid;grid-template-columns:24px 1fr;grid-template-rows:auto;padding:0 8px;row-gap:12px;column-gap:4px}.add[data-v-1b58d296]{flex:0 0 48px;bottom:0;height:48px;display:flex;align-items:center;justify-content:center;border-top:1px solid var(--builderSeparatorColor);background:var(--builderBackgroundColor)}}@layer wf{.BuilderSidebar[data-v-c1777f55]{display:grid;grid-template-rows:1fr 1fr;height:100%}.BuilderSidebar[data-v-c1777f55]>*:not(:first-child){border-top:1px solid var(--builderSeparatorColor)}}
