function __vite__mapDeps(indexes) {
  if (!__vite__mapDeps.viteFileDeps) {
    __vite__mapDeps.viteFileDeps = ["./BuilderSettings-oWe5fV87.js","./index-CUGDV1IA.js","./index-DsXsOFA4.css","./BuilderApp-DVqlivId.js","./marked.esm-273vDTCT.js","./BuilderApp-BNvb4SKa.css","./instancePath-BsbOTTI8.js","./BuilderEmbeddedCodeEditor-BY_NqBZ-.js","./BuilderEmbeddedCodeEditor-DiDqfWdt.css","./BuilderSettings-C5fn5ktN.css"]
  }
  return indexes.map((i) => __vite__mapDeps.viteFileDeps[i])
}
import{d as _,au as V,i as y,a as b,av as h,l as B,c as d,o as n,e as S,M as o,b as c,ag as k,a9 as A,K as M,_ as x}from"./index-CUGDV1IA.js";import{u as C}from"./BuilderSettings-oWe5fV87.js";import{b as j}from"./BuilderApp-DVqlivId.js";import"./instancePath-BsbOTTI8.js";import"./BuilderEmbeddedCodeEditor-BY_NqBZ-.js";import"./marked.esm-273vDTCT.js";const w={class:"BuilderGraphSelect--text"},L=_({__name:"BuilderApplicationSelect",props:V({enableMultiSelection:{type:Boolean,required:!1}},{modelValue:{type:[String,Array],required:!0},modelModifiers:{}}),emits:["update:modelValue"],setup(r){const p=A({loader:()=>M(()=>import("./BuilderSettings-oWe5fV87.js").then(e=>e.a),__vite__mapDeps([0,1,2,3,4,5,6,7,8,9]),import.meta.url),loadingComponent:j}),m=y(b.core),a=h(r,"modelValue"),{load:v,data:s,isLoading:i}=C(m,"applications");B(v);function f(e){switch(e.type){case"chat":return"Chat app";case"framework":return"Framework";case"generation":return"Text generation";case"research":return"Research assistant"}}const g=d(()=>s.value.map(e=>({label:e.name,value:e.id,detail:[f(e),e.status].join(" · ")})).sort((e,t)=>e.label.localeCompare(t.label))),u=d({get(){return a.value===void 0?"":typeof a.value=="string"?a.value:a.value.join(",")},set(e){a.value=e.split(",")}});return(e,t)=>(n(),S("div",w,[o(s).length>0||o(i)?(n(),c(o(p),{key:0,modelValue:a.value,"onUpdate:modelValue":t[0]||(t[0]=l=>a.value=l),options:g.value,"hide-icons":"","enable-search":"",loading:o(i),"enable-multi-selection":r.enableMultiSelection},null,8,["modelValue","options","loading","enable-multi-selection"])):(n(),c(k,{key:1,modelValue:u.value,"onUpdate:modelValue":t[1]||(t[1]=l=>u.value=l)},null,8,["modelValue"]))]))}}),K=x(L,[["__scopeId","data-v-363cd5af"]]);export{K as default};
