@layer wf{.WdsSkeletonLoader[data-v-16706a43]{width:100%;height:12px;background-color:var(--wdsColorGray2);border-radius:8px;background:linear-gradient(90deg,var(--wdsColorGray0) 0%,var(--wdsColorGray2) 50%,var(--wdsColorGray0) 100%);background-size:200% 100%;animation:shimmer-16706a43 1.5s infinite}@keyframes shimmer-16706a43{0%{background-position:100% 0}to{background-position:-100% 0}}}@layer wf{.WdsCheckbox[data-v-555ea1ac]{display:grid;grid-template-columns:auto 1fr;grid-template-rows:auto;align-items:center;column-gap:12px;row-gap:2px;cursor:pointer}.WdsCheckbox__checkbox[data-v-555ea1ac]{border:1px solid var(--wdsColorGray4);width:18px;height:18px;border-radius:4px;grid-row-start:1;grid-row-end:-1}.WdsCheckbox--checked .WdsCheckbox__checkbox[data-v-555ea1ac]{background-color:var(--wdsColorBlue5);font-weight:700}.WdsCheckbox__checkbox__check[data-v-555ea1ac]{display:none}.WdsCheckbox:hover:not(.WdsCheckbox--checked) .WdsCheckbox__checkbox__check[data-v-555ea1ac]{display:block;color:var(--wdsColorGray4)}.WdsCheckbox--checked .WdsCheckbox__checkbox__check[data-v-555ea1ac]{display:block;color:var(--wdsColorWhite)}.WdsCheckbox__detail[data-v-555ea1ac],.WdsCheckbox__label[data-v-555ea1ac]{text-overflow:ellipsis;white-space:nowrap;overflow:hidden;text-align:left}.WdsCheckbox[data-v-555ea1ac]:has(.WdsCheckbox__detail){grid-template-rows:auto auto}.WdsCheckbox__detail[data-v-555ea1ac]{color:var(--wdsColorGray4)}.WdsCheckbox input[data-v-555ea1ac]{display:none}}@layer wf{.WdsDropdownMenu[data-v-a17e8350]{position:absolute;border:1px solid var(--wdsColorGray2);border:none;background:#fff;z-index:2;width:100%;max-height:40vh;overflow-y:auto;border-radius:8px;padding:8px 10px;box-shadow:var(--wdsShadowMenu);box-sizing:border-box}.WdsDropdownMenu[data-v-a17e8350]:has(.WdsDropdownMenu__header){padding-top:0}.WdsDropdownMenu__checkbox[data-v-a17e8350],.WdsDropdownMenu__item[data-v-a17e8350]{min-height:36px}.WdsDropdownMenu__checkbox[data-v-a17e8350]{margin-top:8px;margin-bottom:8px}.WdsDropdownMenu__item[data-v-a17e8350]{background-color:transparent;border:none;display:block;width:100%;display:grid;grid-template-columns:1fr auto;column-gap:8px;align-items:center;border-radius:4px;padding:8px;font-weight:400;font-size:.75rem;color:var(--wdsColorGray6);cursor:pointer;transition:all .2s;pointer-events:all}.WdsDropdownMenu__item[data-v-a17e8350]:has(.material-symbols-outlined){grid-template-columns:auto 1fr auto}.WdsDropdownMenu__item[data-v-a17e8350]:hover{cursor:pointer;background-color:var(--wdsColorBlue1)}.WdsDropdownMenu__item--selected[data-v-a17e8350]{background-color:var(--wdsColorBlue2)}.WdsDropdownMenu__item--hideIcon[data-v-a17e8350]{grid-template-columns:1fr auto}.WdsDropdownMenu__item__detail[data-v-a17e8350],.WdsDropdownMenu__item__label[data-v-a17e8350]{text-overflow:ellipsis;white-space:nowrap;overflow:hidden;text-align:left}.WdsDropdownMenu__item[data-v-a17e8350]:has(.WdsDropdownMenu__item__detail){grid-template-rows:auto auto}.WdsDropdownMenu__item__detail[data-v-a17e8350]{grid-row:2;color:var(--wdsColorGray4)}.WdsDropdownMenu__header[data-v-a17e8350]{position:sticky;top:0;padding-top:8px;padding-bottom:8px;background:#fff;display:flex;flex-direction:column;gap:8px}.WdsDropdownMenu__header__search[data-v-a17e8350]{background-color:var(--wdsColorGray1);border-radius:4px;height:36px;width:100%;display:flex;align-items:center;justify-content:center;gap:4px;padding:8px;border:1px solid transparent;color:currentcolor}.WdsDropdownMenu__header__search[data-v-a17e8350]:hover{border-color:var(--wdsColorBlue1)}.WdsDropdownMenu__header__search[data-v-a17e8350]:focus-within{background-color:#fff;outline:4px solid var(--wdsColorBlue1)}.WdsDropdownMenu__header__search__input[data-v-a17e8350]{letter-spacing:inherit;color:currentcolor;padding:4px 0 5px;border:0px;box-sizing:content-box;background:none;height:1.4375em;margin:0;display:block;min-width:0px;width:100%;animation-name:mui-auto-fill-cancel;animation-duration:10ms;-webkit-appearance:textfield;-moz-appearance:textfield;appearance:textfield}.WdsDropdownMenu__header__search__input[data-v-a17e8350]:focus-visible{outline:none}.WdsDropdownMenu__header__multiSelect[data-v-a17e8350]{display:flex;gap:12px}.WdsDropdownMenu__header__multiSelect__count[data-v-a17e8350],.WdsDropdownMenu__header__multiSelect__clear[data-v-a17e8350]{text-transform:uppercase;font-size:13px}.WdsDropdownMenu__header__multiSelect__count[data-v-a17e8350]{flex-grow:1;color:var(--wdsColorGray6)}.WdsDropdownMenu__header__multiSelect__clear[data-v-a17e8350]{background-color:transparent;border:none;cursor:pointer;text-align:right;color:var(--wdsColorGray4)}}
