import{_ as t,o as a,e as s,f as o}from"./index-CUGDV1IA.js";const r={},d={class:"CoreChatbotLoader",style:{"--dotSize":"6px"}};function n(_,e){return a(),s("div",d,e[0]||(e[0]=[o("span",{class:"CoreChatbotLoader__dot"},null,-1),o("span",{class:"CoreChatbotLoader__dot"},null,-1),o("span",{class:"CoreChatbotLoader__dot"},null,-1)]))}const l=t(r,[["render",n],["__scopeId","data-v-06a14934"]]);export{l as default};
